@@ -1,9 +1,11 @@
 from __future__ import annotations
+from inspect import isgenerator
 
-from typing import Dict
+from typing import Any, Callable, Dict, List
+import itertools
 from violetear.color import Color, Colors
 from violetear.stylesheet import StyleSheet
-from violetear.units import Unit, rem
+from violetear.units import Unit
 
 # ## Flex-based grid system
 
@@ -128,3 +130,78 @@ class SemanticDesign(StyleSheet):
         self.buttons()
 
         return self
+
+
+# ## Utility system
+
+
+class UtilitySystem(StyleSheet):
+    def __init__(self) -> None:
+        super().__init__()
+
+    def define(
+        self,
+        rule: str,
+        cls: str,
+        variants: List[str],
+        values: List[Any] = None,
+        fn: Callable = None,
+        variant_name: Callable = None,
+    ):
+        if isinstance(variants[0], (list, tuple)) or isgenerator(variants[0]):
+            variants = list(itertools.product(*variants))
+
+        if values is None:
+            values = variants
+
+        if fn is not None:
+            fn_values = []
+
+            for value in values:
+                if isinstance(value, tuple):
+                    result = fn(*value)
+                else:
+                    result = fn(value)
+
+                fn_values.append(result)
+
+            values = fn_values
+
+        for variant, value in zip(variants, values):
+            if variant_name is not None:
+                if isinstance(variant, tuple):
+                    variant = variant_name(*variant)
+                else:
+                    variant = variant_name(variant)
+
+            elif isinstance(variant, tuple):
+                variant = "-".join(str(x) for x in variant)
+
+            self.select(f".{cls}-{variant}").rule(rule, str(value))
+
+        return self
+
+    def define_many(
+        self,
+        rule: str,
+        subrules: List[str],
+        clss: List[str],
+        variants: List[str],
+        values: List[Any] = None,
+        fn: Callable = None,
+        variant_name: Callable = None,
+    ):
+        variants = list(variants)
+        values = list(values)
+
+        for subrule, cls in zip(subrules, clss):
+            self.define(
+                rule=f"{rule}-{subrule}",
+                cls=cls,
+                variants=variants,
+                values=values,
+                fn=fn,
+                variant_name=variant_name,
+            )
+
+        return self
